	/* Standard C Includes */
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <sys/stat.h>
#include <string.h>
#include <stdarg.h>
#include <dos.h>
#include <process.h>
#include <dir.h>

	/* Some Personal Include Files */
#include "genutil.h"
#include "fileutil.h"

	/* File pointer for the log file. I have an open file throughout the
	 * programs operation to record various important events. This may help
	 * identify any bugs/errors which are occuring */
FILE *log_file;
boolean log_file_open = FALSE;		/* Is the log file currently open? */


//========= Checks to see if there is any disk in drive =====================
boolean check_disk (const int drive) {
	/* If one attempts to get the filelist in a floppy drive with no disk
	 * a DOS error occurs and really messes things up. Use this function
	 * to check any drive for a valid disk. If change was successfull the
	 * current drive is changed to the specified drive */

  int save_disk;
  struct find_t file_block;

	/* Save the current drive and attempt to change drives */
  save_disk = getdisk();
  setdisk (drive);

	/* Try and find files in drive...no files means no disk. If there is
	 * an error hopefully the DOS error handler will be called with the
	 * familiar 'Ignore, Retry, Abort' message. Return value appropiately */
  switch (_dos_findfirst("*.*", FA_DIREC, &file_block)) {
    case DOS_ERR_ABORT:	 setdisk (save_disk); return (FALSE);
    case DOS_ERR_IGNORE: return (FALSE);
    case DOS_ERR_RETRY: return (check_disk(drive));
   }
	/* Change Was succesful */
  return (TRUE); }
//========= End of function check_drive() ===================================


//========== Closes the Log File ============================================
void close_log_file (void) {
	/* Closes the log file, if it is open, and sets the boolean variable */
  write_log_entry ("Closing log file...");
  if (log_file_open) fclose (log_file);	/* Close the file if needed */
  log_file_open = FALSE;
 }
//========== End of procedure open_log_file () ==============================


//========== Some sort of FILE_IO error =====================================
void file_bug(char *msg, char *string, ...) {
	/* Simple sets the proper error header strings and calls the general
	 * bug/error handler. This procedure should only be called by functions
	 * and procedures in this file. */
  err_header_string = FILEUTIL_STRING;
  err_header_version = FILEUTIL_VERSION;
  bug (msg, string, ...);
 }
//========== End of procedure file_bug() ====================================


//========== Returns the length in bytes of specified filename ==============
long get_filesize(const char *filename) {
	/* This function returns the filesize of a un-opened file. The file
	 * may, in fact, be opened, but this function opens the file
	 * temporarially in 'read-only' mode to get the filesize. Will not
	 * return if there is an error opening the file */

  FILE *f;	/* File Pointer, since we must open file to get info */
  struct stat file_stat;	/* File statistics structure to hold required information */

  f = openfile (filename, "r");	/* Attempt to Open file */
  fstat(fileno(f), &file_stat);	/* Read file statistics */
  fclose(f);			/* Close File */

	/* Return filesize...pretty painless huh? */
  return(file_stat.st_size);
 }
//========= End of function get_filesize() ==================================


//========== Returns TRUE if file exists ====================================
boolean fileexists(const char *filename) {
	/* This function returns TRUE if the speficied filename exists. It
	 * checks this by attempting to open the file in read-only mode. If
	 * the open succeeds, the file must exist */

  FILE *f;	/*File pointer */

	/* Attempt to open file for reading. Don't use my function openfile()
	 * this this will exit() the program if the file can't be opened */
  if ((f = fopen(filename, "r")) == NULL)
    return(FALSE);	/* File couldn't be opened...return failure */

  fclose(f); 	/* File was opened and therefore exists, close it */
  return(TRUE);	/* Return success */
 }
//========== End of function fileexists() ===================================


//========== Returns the length in bytes of specified filename ==============
long get_filesize(const FILE *f) {
	/* This function, like the above one, returns the size of a file.
	 * the only different is that this one assumes the file is already
	 * and uses the files pointer instead of it's name to get the size
	 * information */

  struct stat file_stat; 	/* File statistics structure to hold required information */

  fstat (fileno(f), &file_stat);	/* Read file statistics */

	/* Return the file size */
  return (file_stat.st_size);
 }
//========= End of function get_filesize() ==================================


//========== Checks to see if a filename is a directory or not and changes directory
boolean isdirectory(const char far *string) {
  char buffer[13], *temp_ptr;

	//Only take 1st 12 characters...valid file/path name
  strncpy (buffer, string, 12);
  buffer[12] = 0; 	//Make sure string is terminated

	//Remove trailing spaces from path
  temp_ptr = &buffer[strlen(buffer) - 1];

  while (*temp_ptr == ' ' && strlen(buffer) > 0) {
    *temp_ptr = 0;
    temp_ptr --;
   }

  if (chdir(buffer))
    return(FALSE);
  else
    return(TRUE);
 }
//========== End of function isdirectory() ==================================


//========== Checks if a string has any '*' or '?' in it ====================
boolean iswildcard(const char far *string) {

  for (;*string != '\0'; string++) {
    if (*string == '*' || *string == '?') return(TRUE);
   }

  return(FALSE); }
//========== End of function iswildcard() ===================================



/*========== Attempts to open filename in Given Mode ======================*/
FILE *openfile (const char *filename, const char *mode) {
	/* Instead of checking the file pointer each time I opened a file, I
	 * created this function to do this for me. I use it exactly like the
	 * standard 'C' function fopen(). The only difference is that it calls
	 * the bug() routine and exits the program on an error. If you don't
	 * want to exit on an error, check the filename with the fileexists()
	 * function above, or simply use the fopen() */
  FILE *f;	/* File pointer */

	/* I would like to notify the operator I'm opening a file */
  write_log_entry ("Attempting to open file '%s' in mode '%s'", filename, mode);

	/* Attempt to open the filename in the given mode */
  if (!(f = fopen (filename, mode)))
    file_bug ("Bad filename or file does not exist", "Filename = '%s'\n   Mode = '%s'", filename, mode);

	/* A successful open, return the file pointer */
  return (f);
 }
//========== End of Function openfile() =====================================


//========== Opens a Log File ===============================================
void open_log_file (const char *filename) {
	/* This procedure opens a log file which contains messages written
	 * in order to trace any bugs/errors/crashes. This file remains open
	 * until the program is exited, or close_log_file() is called. close_log_file()
	 * is automatically posted as a exit routine below. Use the procedure
	 * write_log_entry() to access the log_file. The default log filename
	 * is 'logfile.txt'. If no filename is supplied this one is used (overwriting
	 * any previous version of logfile.txt */

  struct date d;	/* The current date */

	/* Attempt to open file and set file open boolean variable*/
  log_file = openfile (filename, "wt");
  log_file_open = TRUE;

	/* Get the current Date and write it to file */
  getdate (&d);
  write_log_entry ("Opened Log File %s on %d/%d/%d", filename, d.da_day, d.da_mon, d.da_year);

	/* Make sure we close log file on exit and write another log entry*/
  atexit (close_log_file);
  write_log_entry ("Registered Exit Function close_log_file()");
 }
//========== End of procedure open_log_file () ==============================


//========== Attempts to Read a file until a CR or EOF is encountered =======
boolean read_eol(FILE *f, char far *buf, const int msl) {
	/* I often need to read in ASCII type text files such as configuration
	 * or text data files. It is conveinient to read until the end of the
	 * current line, which this program does. You must provide a pre-allocated
	 * pointer to place the data, as well as a previously open file pointer.
	 * This function will only read a maximum of max_string_length characters
	 * which is useful to avoid writing past the end of allocated memory (which
	 * can cause some very nasty stuff...believe me). It will return TRUE unless
	 * it reads an EOF (end of file) marker, allowing one to create a useful
	 * read loop.
	 * msl = The maximum allowed string length. If not specified it is
	 * equal to the variable max_string_length */
  int ch;	/* Temp character buffer */
  int i = 2;

  ch = fgetc(f);	/* Get the 1st character from file */

	/* Set supplied string to initially nothing, but only if a
	 * string is given to us */
  if (buf) *buf = '\0';

	/* Loop until an EOF marker is found, a Carriage Return is found or
	 * the loaded string length exceeds the maximum allowable */
  while(ch != EOF && ch != CR && i < msl) {
	/* If buffer is NULL, this means that we want to read to the end of
	 * the current line in a file, ignoring all the data on that line */
    if (buf) {
      chrcat(buf, (char) ch);
      i++;
     }

    ch = fgetc(f);	/* Read the next character */
  }

	/* If an EOF marker was read...return FALSE */
  if (ch == EOF) return(FALSE);

	/* Otherwise, return success */
  return(TRUE); }
//========== End of function read_eol() =====================================


//========== Attempts to Read a file until a Space is encountered ===========
boolean read_word(FILE *f, char far *buf) {
	/* This is much like the function read_eol() above, except that it reads
	 * a word, instead of a line. A word is a defined (by me) as something
	 * which is between to space characters (ASCII code = 32). As above,
	 * returns FALSE if an EOF marker was found and will not read past
	 * the max_string_length */
  int ch;	/* Temp input character */
  int i = 1;

  ch = fgetc(f);	/* Read in 1st character from file */

	/* Set supplied string, if any, to initially nothing */
  if (buf) *buf = '\0';

	/* Ignore initial spaces */
  while (ch == ' ' && ch != EOF && ch != CR)
    ch = fgetc(f);

	/* Loop until a space is read, a Carriage return is read, an EOF
	 * marker is read, or max_string_length characters have been read */
  while (ch != ' ' && ch != EOF && ch != CR && ch != 0xFF && i < max_string_length - 1) {
	/* Add the string buffer, but only if it was supplied */
    if (buf) {
      chrcat(buf, (char) ch);
      i++;
     }

    ch = fgetc(f);	/* Get the next character from the file */
   }

	/* Return FALSE if we read an EOF marker */
  if (ch == EOF || ch == 0xFF) return(FALSE);

	/* If last character read was a CR, add to buffer, if buffer has
	 * enough room to put it */
  if (ch == CR && strlen(buf) < max_string_length - 1) chrcat (buf, (char)ch);

	/* Return a success */
  return(TRUE); }
//========== End of function read_eol() =====================================



//========== Reads in Entire File and Returns Pointer to Allocated Mem ======
char far *read_file (const char *filename) {
	/* I often need to read in an entire file and put it into memory
	 * such as palettes, fonts, etc... I can use this function to do
	 * exactly that...with no worry about allocating memory etc...
	 * returns a far pointer to the file data and calls a bug routine
	 * on an error. This can only be used to read files whose filesize
	 * can 'fit' in an unsigned integer. Will return error if not.
	 * This is because the new operator only accepts unsigned short
	 * values...too bad, but this shouldn't be too much of a problem
	 * for 99% of cases */

  FILE *f;	/* The file Pointer */
  char far *data_ptr, *temp_ptr;	/* Temp pointers */

	/* Attempt to Open file in read-only binary mode */
  f = openfile (filename, "rb");

	/* Attempt to allocate memory for file, making sure the filesize is
	 * in the allowed range */
  if (get_filesize(f) > 65529l)
    file_bug ("ERROR: *read_file() can only read files less than 65529b in size!", "Filename = '%s' (%d)", filename, get_filesize(f));


  if (!(data_ptr = new char[(unsigned int) get_filesize (f) + 16]))
    file_bug (MEM_ERR_MSG, "Filename = '%s' (%d)", filename, get_filesize (f) + 16);

	/* Point pointer to 16-byte paragraph offset. This is because I
	 * occasionally need these pointers to call various functions
	 * (such as the ct_voice.drv). If these functions aren't on an
	 * 16-byte paragraph offset boundary, the program _will_ crash.
	 * That is why I allocated an extra 16 bytes for the file above */
  if (FP_OFF (data_ptr) != 0)  {
    FP_OFF (data_ptr) = 0;
    FP_SEG (data_ptr)++;
   }

	/* Read in the file, 1kb at a time. Don't ask me why I do
	 * it this way... */
  temp_ptr = data_ptr;

  while (fread (temp_ptr, sizeof (char), 1000, f) == 1000) {
    temp_ptr += 1000;
   }

	/* Close file, but then again, if I really needed to tell you that,
	 * you shouldn't be looking at this at all...;) */
  fclose (f);

	/* Return the pointer to the load file data */
  return (data_ptr);
 }
//========== End of Function read_file() ====================================


//========== Reads an Integer from a file ===================================
int read_int (FILE *f) {
	/* Reads in an integer from a file. As any hex-editting guru can tell
	 * you, the byte sequence for integers in files are reversed. i.e.,
	 * a number '4E 6A' in a file is actually 0x6A4E = 27,214. I don't
	 * trust scanf() so I simply read 2 character bytes, and convert them
	 * to integers using the HexToInt() define functions. This assumes the
	 * file is already open, and returns -1 on EOF */
  unsigned char buffer[2];	/* Temp input buffer */

	/* Assumes file is already open...read 2 characters */
  if (fread(&buffer[0], sizeof(char), 2, f) != 2) return(-1);

	/* Convert to integer and return value */
  return (HexToInt(&buffer[0]));
 }
//========== End of function read_int() =====================================


//========== Reads a Long Integer from a file ===============================
long read_long (FILE *f) {
	/* Same function as above (read_int()), except for long integers */

  unsigned char buffer[4];	/* Temp input buffer */

	/* Assumes file is already open...read 4 bytes */
  if (fread(&buffer[0], sizeof(char), 4, f) != 4) return(-1);

	/* Convert to long integer and return */
  return (HexToLong(&buffer[0]));
 }
//========== End of function read_long() ====================================


//========== Writes Integer to File =========================================
boolean write_int (FILE *f, const unsigned i) {
	/* Read comments in procedure read_int() for more info. Writes an
	 * integer to an open file, with the correct byte sequence. Returns
	 * false if the write to the file fails */
  char buffer[3];	/* Temp string */

	/* Convert integer to hex string and attemp to write to file */
  int_to_hex(i, &buffer[0]);
  if (fwrite(&buffer[0], sizeof(char), 2, f) != 2) return (FALSE);

	/* Return a successful write */
  return (TRUE);
 }
//========== End of function write_int() ====================================


//========== Writes an Entry to the Log File, if Opened =====================
void write_log_entry (char *string, ...) {
	/* Writes a 'line' to the log_file. Will not write entry if file has
	 * not been opened by a call to open_log_file() or if file has since
	 * been closed. Accepts arguments exactly like a printf() function. */

  struct time t;	/* The current time */
  va_list args;		/* For the variable arugment list */

	/* Don't write anything if file isn't opened */
  if (!log_file_open) return;

	/* Write the string to the log file..if any supplied */
  if (string != NULL) {
	/* Get the current Time and write to file has a header text */
    gettime (&t);
    fprintf (log_file, "%2d:%02d:%02d.%02d - ", t.ti_hour, t.ti_min, t.ti_sec, t.ti_hund);

	/* Print the arugment list to file */
    va_start (args, string);
    vfprintf (log_file, string, args);
    va_end (args);

	/* A linefeed */
    fprintf (log_file, "\n");

	/* Flush the file stream to make sure the written characters are written
	 * to the file. This is in case the program crashes and isn't closed. If
	 * this did happen without the following line, we'd probably lose some data. */
    fflush (log_file);
   }

 }
//========== End of procedure write_log_entry() =============================


//========== Writes Long Integer to File ====================================
boolean write_long (FILE *f, const unsigned long l) {
	/* Same as function write_int() except this is for long integers.
	 * Returns FALSE on a file write error */

  char buffer[5];	/* Temp string buffer */

	/* Convert integer to hex-string and write 4 bytes to file */
  long_to_hex(l, &buffer[0]);
  if (fwrite(&buffer[0], sizeof(char), 4, f) != 4) return (FALSE);

	/* Returns a successful write */
  return (TRUE);
 }
//========== End of function write_long() ===================================
