	/* Some Standard 'C' Include Files */
#include <stdlib.h>
#include <conio.h>
#include <stdio.h>
#include <string.h>
#include <dos.h>
#include <alloc.h>
#include <stdarg.h>
#include <ctype.h>

	/* Some Personal Include Files */
#include "genutil.h"
#include "fileutil.h"

	/* Used to hold old interrupt vector, i.e., the original DOS Timer
	 * interrupt routine. This is saved by set_timer_int() and used again
	 * in  remove_timer_int ()*/
void interrupt (*Old_Time_Isr)(__CPPARGS);

	/* Have we saved the Original DOS Timer Interrupt yet? This prevents
	 * us from replacing it before saving it (which would crash the program
	 * pretty fast, or from saving the modified Timer Interrupt routine
	 * instead of the original one */
boolean Old_Timer_Saved = FALSE;

	/* Some global strings for error handling. These can be used to
	 * identify the program calling the error handling routine. */
char far *err_header_string;
char far *err_header_version;

	/* Global Error codes. These are used to pass error codes from one procedure
	 * or function to another where the codes are parsed/displayed. For
	 * example, in a function an error occurs. The error code variables
	 * below are set and the function returns failure (FALSE).   */
unsigned err_code, ext_err_code;

	/* Global, Maximum allowable string length. Is used to check bounds
	 * in several string/input routines. I made it a variable instead of
	 * a constant/define since I may want different sized strings in
	 * different programs and I want to be flexible */
unsigned int max_string_length = GENUTIL_STRING_LENGTH;

	/* List of functions to call. This array contains a list of procedures
	 * the new timer interrupt must call when invoked. See the header file
	 * for more detail. */
TASK_TYPE task_list[MAX_TASKS];
unsigned num_tasks = 0;		/* Number of tasks in task_list[] */
boolean do_tasks = FALSE;   	/* Are we processing all the tasks currently? */

	/* Several Error codes descriptions I use regularily. Accessed by
	 * the error code define statements in header file */
char far *err_desc[] =
  { "Read past End of File",
    "Invalid File Type",
    "Not Enough Data To Read",
    "Overflow error (array, pointer, or numerical)",
    "\0" };


//========= Adds a Task to the Task List ====================================
int add_task (void far (*function)(__CPPARGS), const unsigned max_counter) {
	/* Returns -1 if fails, otherwise the returns index in the task_list[]
	 * array. Use this index to delete the task */

	/* Better write a log entry in case this procedure crashes when
	 * called by the timer interrupt procedure */
  write_log_entry ("Add task %d of %d to the task list (max_counter = %d)", num_tasks, MAX_TASKS, max_counter);

	/* If a NULL function was specified...better write something */
  if (!function) write_log_entry ("ERROR: A NULL task procedure was added to the task_list[] array!");

	/* Make sure we haven't exceeded the task_list[] array bounds */
  if (num_tasks > MAX_TASKS) return (-1);

	/* Set the initial values of the task */
  task_list[num_tasks].active = TRUE;
  task_list[num_tasks].task = function;
  task_list[num_tasks].max_count = max_counter;
  task_list[num_tasks].count = 0;

	/* Increase number of current tasks in array and return task index */
  num_tasks++;
  return (num_tasks - 1);
 }
//========= End of function add_task () =====================================


//========= Notifies User of a bug/error ====================================
void bug(char *msg, char *string, ...) {
  /* General error message routine notifying user of an error. Uses the variable
   * arguments vsprintf () function to emulate the printf (...) function. This
   * allows my to treat it exactly as I would a printf, except for the initial
   * msg string. */

  va_list args;		/* The variable argument list */

	/* Return to text mode...hopefully. This is just in case we happen
	 * to be in some Mode X video mode */
  asm {
    mov   ax, 3
    int   0x10
   }

  textmode (C80);

	/* Clear the screen and print the program header and version, if they
	 * are specified */
  clrscr ();
  if (err_header_string) printf (err_header_string, err_header_version);
  printf ("An error has occured.  You should record this information along with\n");
  printf ("conditions before the crash (filenames, actions, etc...), system info\n");
  printf ("(CPU type, OS, memory, etc...) and send a bug report to...\n");
  printf ("                 aj589@freenet.carleton.ca\n\n");

	/* Print various system information, like available memory etc...
	 * this may help us find out the cause of the bug/error */
  printf ("farheapcheck() = %1d   farcoreleft = %ld\n", farheapcheck(), farcoreleft());
  if (errno < sys_nerr) printf ("System Error Code = %s (%d)\n\n", sys_errlist[errno], errno);

	/* The the main bug message */
  printf ("BUG: %s\n", msg);

	/* The the 2nd string which is the printf... formatted one. Only
	 * print it if it is included */
  if (string) {
	/* Print a little spacing header */
    printf ("    ");

	/* Print the arugment list to the screen */
    va_start (args, string);
    vprintf (string, args);
    va_end (args);
   }

	/* Print a few padding linefeeds and exit the program without calling
	 * any posted exit procedures. Calling these procedures might do
	 * weird things since an exception (bug/error) has just occured */
  printf ("\n\n\n");
  _exit (FALSE);
 }
//========= End of procedure bug() ==========================================


/*========= Attempts to Create a pointer ==================================*/
char *create_ptr (const char *string, char *ptr) {
  char *temp_ptr;

  if (ptr) {
    delete ptr;
    ptr = NULL;
   }

  if (!string) return (NULL);

  if (!(temp_ptr = new char[strlen(string) + 1]) )
    bug (MEM_ERR_MSG, "create_ptr() - *temp_ptr (%d)", strlen(string) + 1);

  strcpy (temp_ptr, string);
  return (temp_ptr); }
/*========= End of function create_ptr() ==================================*/


/*========= Attempts to Create a pointer ==================================*/
char *create_ptr (const unsigned l, char *ptr) {
  char *temp_ptr;

  if (ptr) {
    delete ptr;
    ptr = NULL;
   }

  if (!(temp_ptr = new char[l]) )
    bug (MEM_ERR_MSG, "create_ptr() - *temp_ptr (%d)", l);

  return (temp_ptr); }
/*========= End of function create_ptr() ==================================*/


/*========= Attempts to Create a far pointer ==============================*/
char far *create_farptr (const char far *string, char far *ptr) {
  char far *temp_ptr;

  if (ptr) {
    delete ptr;
    ptr = NULL;
   }

  if (!string) return (NULL);

  if (!(temp_ptr = new far char[_fstrlen(string) + 1]) )
    bug (MEM_ERR_MSG, "create_farptr() - *temp_ptr (%d)", _fstrlen(string) + 1);

  _fstrcpy (temp_ptr, string);
  return (temp_ptr); }
/*========= End of function create_farptr() ===============================*/



/*========= Attempts to Create a far pointer ==============================*/
char far *create_farptr (const unsigned l, char far *ptr) {
  char far *temp_ptr;

  if (ptr) {
    delete ptr;
    ptr = NULL;
   }

  if (l <= 0) return (NULL);

  if (!(temp_ptr = new far char[l]) )
    bug (MEM_ERR_MSG, "create_farptr() - *temp_ptr (%d)", l);

  *temp_ptr = 0;
  return (temp_ptr); }
/*========= End of function create_farptr() ===============================*/



/*========= Appends a character to end of string ==========================*/
void chrcat(char far *buf, const char ch) {
	/* Just a simple procedure which I use a lot. Very handy in parsing
	 * situations where each input/output character needs to be examined.
	 * Be careful not to add a character past end of string
	 * since array bounds overflow can easily happen and corrupt data (i.e.,
	 * writing characters past end of allocated string...ugh!) */
  int i = _fstrlen(buf);	/* Define a constant so we don't have to compute it twice */

	/* Add the character to end of string, making sure we NULL pad the
	 * end of the string */
  *(buf + i + 1) = '\0';
  *(buf + i) = ch; }
/*========= End of procedure chrcat() =====================================*/


/*========= Prints Text Center on the Screen...by some offset =============*/
int cprintf_cent (const int y_offset, const char far *format, ...) {
	/* I often like to print a line centered x and y-wise on the screen.
	 * This function does exactly that, taking the exact same format as any
	 * standard printf() function except for the y_offset. The y_offset
	 * prints the text a certain distance from the y-center of the screen.
	 * Be careful not to print exceedingly large strings as this will overflow
	 * the string array */

  va_list arg_list;
  char buffer[GENUTIL_STRING_LENGTH];

	/* Print the text to string buffer */
  va_start(arg_list, format);
  vsprintf(&buffer[0], format, arg_list);
  va_end(arg_list);

	/* Move to appropiate position in the screen */
  if (get_screenheight() > 25)
    gotoxy ((get_screenwidth() - _fstrlen(buffer))/2 + 1, get_screenheight()/2 + y_offset);
  else
    gotoxy ((get_screenwidth() - _fstrlen(buffer))/2 + 1, get_screenheight()/2 + y_offset + 1);

	/* Print the text to the screen */
  return (cprintf ("%s", buffer));
 }
//========== End of function cprint_cent() ==================================


//========== Deletes a Task from the Task List Array ========================
void delete_task (unsigned i) {
	/* Removes a task with index 'i' from the task_list[] array. Will not
	 * remove a task if it doesn't exist (out of current task bounds) or if
	 * there are no tasks in list. */

	 /* Turn task processing off while we move things around. This prevents
	  * the timer interrupt routine from being called while we delete and
	  * move the task_list[] array items...if it did, it could crash the
	  * program */
  do_tasks = FALSE;

	/* Write log entry, just in case ...*/
  write_log_entry ("Deleting task #%d", i);

	/* Make sure index is in bounds. Don't need to check lower bounds since
	 * i is an unsigned integer */
  if (i >= num_tasks) return;

	/* Shift tasks */
  for (; i < num_tasks; i++) {
    task_list[i].active = task_list[i + 1].active;
    task_list[i].task = task_list[i + 1].task;
   }

	/* Decrease number of tasks and Turn task processing back on. Pretty
	 * painless huh? */
  num_tasks--;
  do_tasks = TRUE;
 }
/*========== End of procedure delete_task () ==============================*/


/*========== Returns Current Background Color in Text Mode ================*/
int get_back_color(void) {
  return (get_textattr() >> 4);
 }
/*========== End of function get_back_color() =============================*/


/*========== Returns the background color at a screen location ============*/
int get_char_backcolor(const int x, const int y) {
  char buffer[3];

  gettext (x, y, x, y, buffer);
  return (buffer[1] >> 4);
 }
/*========== End of Function get_char_backcolor() =========================*/

/*========== Returns the text color at a screen location ==================*/
int get_char_textcolor(const int x, const int y) {
  char buffer[3];

  gettext (x, y, x, y, buffer);
  return (buffer[1] & 15);
 }
/*========== End of Function get_char_textcolor() =========================*/


/*========== Reads a line of text from the Keyboard =======================*/
boolean get_line (char far *buffer, const int msl) {
	/* Returns TRUE if user presses ENTER. FALSE if input exceeds msl characters
	 * or if ESC was pressed. msl = max_string_length if not specified */
  char ch;
  int x = wherex(), y = wherey();

  *buffer = 0;	/* Reset string */

  do {
    gotoxy (x, y);
    printf ("%s ", buffer);
    gotoxy (x + _fstrlen(buffer), y);

    ch = getch();

    if (ch >= ' ' and ch <= '~')
      chrcat (buffer, ch);
    else if (ch == 8 and _fstrlen (buffer) > 0)
      buffer[_fstrlen(buffer) - 1] = 0;

    if (_fstrlen(buffer) >= msl or ch == 27) return (FALSE);
  } while (ch != 13);


  return (TRUE);
 }
/*========== End of function get_line() ===================================*/


/*========== Returns the Height in Characters of current text screen ======*/
int get_screenheight(void) {
  struct text_info ti;

  gettextinfo (&ti);

  return (ti.screenheight); }
/*========== End of function get_screenheight() ===========================*/


/*========== Returns the Width in Characters of current text screen =======*/
int get_screenwidth(void) {
  struct text_info ti;

  gettextinfo (&ti);

  return (ti.screenwidth); }
/*========== End of function get_screenwidth() ============================*/


/*========== Returns Current Text Color in Text Mode ======================*/
int get_text_color(void) {
  return (get_textattr() & 15); }
/*========== End of function get_text_color() =============================*/


/*========== Returns Current Text Attribute in Text Mode ==================*/
int get_textattr(void) {
  struct text_info tmp;
  gettextinfo(&tmp);
  return (tmp.attribute); }
/*========== End of function get_textattr() ===============================*/


/*========== Gets a Yes/No type answer from the keyboard ==================*/
int get_yesno (void) {
  char ch;

  do {
    ch = tolower(getch());
  } while (ch != 'y' && ch != 'n' && ch != 27);

  if (ch == 'y')
    return (CH_YES);
  else if (ch == 'n')
    return (CH_NO);
  else
    return (CH_ESC);
 }
/*========== End of function get_yesno() ==================================*/


/*========== Gets a Yes/No/All type answer from the keyboard ==================*/
int get_yesnoall (void) {
  char ch;

  do {
    ch = tolower(getch());
  } while (ch != 'y' && ch != 'n' && ch != 27 && ch != 'a');

  if (ch == 'y')
    return (CH_YES);
  else if (ch == 'n')
    return (CH_NO);
  else if (ch == 'a')
    return (CH_ALL);
  else
    return (CH_ESC);
 }
/*========== End of function get_yesno() ==================================*/



//========== Converts a Int to Hexadecimal String ===========================
void int_to_hex (const unsigned int i, char *buf) {
	/* As any accomplised hex-editting guru can tell you, the two integer
	 * bytes in files are reversed. ex: If an integer written in a file
	 * appears like '1A 6F' the actual value is 0x6F1A = 28,442
	 * I created this function since I don't trust using the printf()
	 * functions to write things. I'd rather convert an integer using this
	 * procedure and write two character bytes. Pretty simple to
	 * do if you're familiar with number base conversion...otherwise */
  unsigned int rem, div;

  rem = i/256;
  div = i - rem * 256;
  *(buf + 1) = (unsigned char) rem;
  *(buf) = (unsigned char) div;
  *(buf + 2) = '\0';
 }
//========== End of procedure int_to_hex() ==================================


//========== Is a Point (x,y) in a particular Rectangle? ====================
boolean iscontained(int x, int y, int x1, int y1, int width, int height) {
  if (x >= x1 && y >= y1 && x <= x1 + width && y <= y1 + height - 1)
    return(TRUE);
  else
    return(FALSE);
 }
//========== end of function iscontained() ==================================


//========== Returns true if ch is an alphanumeric character ================
boolean isprintable(const char ch) {
	/* Returns TRUE if the character is a printable character */

  if (ch >= ' ' && ch <= '~')
    return (TRUE);
  else
    return (FALSE);

 }
//========== End of function isprintable() ==================================


//========== Returns TRUE if character is a Punctuation Char ================
boolean ispunctuation(const unsigned char ch) {
	/* Is the character a punctuation type character? */

  if (ch == '.' || ch == ',' || ch == ';' || ch == ':' || ch == ' '  ||
      ch == '?' || ch == '!' || ch == '!' || ch >= 0xFC || ch == ')' ||
      ch == '(' || ch == '[' || ch == ']' || ch == '-' || ch == '\'' ||
      ch == '`' || ch == '"')
    return(TRUE);

  return(FALSE); }
//========== End of function ispunctuation() ================================


//========== Converts a Long Int to Hexadecimal String ======================
void long_to_hex (const unsigned long i, char *buf) {
	/* Same as the function int_to_hex() above, except that it does it
	 * for a long integer */
  unsigned long rem, div;

  rem = i/256/256/256;
  div = i - rem * 256 * 256 * 256;
  *(buf + 3) = (unsigned char) rem;

  rem = div/256/256;
  div = div - rem * 256 * 256;
  *(buf + 2) = (unsigned char) rem;

  rem = div/256;
  div = div - rem * 256;
  *(buf + 1) = (unsigned char) rem;
  *(buf) = (unsigned char) div;
  *(buf + 4) = '\0';
 }
//========== End of procedure long_to_hex() =================================


/*========== Strips Space and TAB characters from Left of String Returns ====
  Pointer to 1st non-space/tab character in string ========================*/
char *lstrip (char *string) {

  while (string && *string && (*string == ' ' || *string == 9)) {
    string++;
   }

  return (string); }
/*========== End of function lstrip() =====================================*/


//========== Returns the maximum of two values ==============================
unsigned maximum (unsigned value1, unsigned value2) {
	/* Simply returns the maximum of two values. A much better and more flexible
	 * routine would have a variable number of arguments...but since I don't
	 * currently need it... */
  return ( (value1 > value2) ? value1 : value2 );
 }
//========== End of function maximum() ======================================


/*========== Parses a Command Line Command =================================*/
boolean parse_command (char *string, COMMAND_TYPE *cmd_table) {
  char *temp_ptr1, *temp_ptr; 	/* Temp string buffers */
  COMMAND_TYPE *start_cmd = cmd_table;

  if (!string) return (FALSE);
  temp_ptr = lstrip(string);
  temp_ptr = rstrip(temp_ptr);

  while (cmd_table->name and cmd_table->do_command) {
    if (ISEQ(temp_ptr, cmd_table->name) || ISEQ(cmd_table->name, temp_ptr)) {
      temp_ptr = lstrip(string);

      if ((temp_ptr1 = strchr(temp_ptr, ' ')) != NULL)
	temp_ptr1 = lstrip(temp_ptr1);

      cmd_table->do_command (temp_ptr1);
      return (TRUE);
     }

    cmd_table++;
   } /* End of while loop */

  if (strlen(temp_ptr) == 0) return (FALSE);

	/* Display the possible commands */
  printf ("\n   Possible commands are...");

  while (start_cmd->name && start_cmd->do_command) {
    printf ("\n      %s", start_cmd->name);
    start_cmd++;
   }

  return (FALSE);
 }
/*========== End of Function parse_command() ===============================*/


/*========== Prints a comma delimited number ===============================*/
void print_number (const unsigned long num) {
  unsigned long i = num;

  if (num >= 1e9) {
    printf ("%d,", (unsigned) (num/1e9));
    i -= ((unsigned long)(num/1e9)) * 1e9;
   }

  if (i >= 1e6) {
    if (num > 1e9)
      printf ("%03d,", (unsigned)(i/1e6));
    else
      printf ("%d,", (unsigned)(i/1e6));

    i -= ((unsigned long)(i/1e6)) * 1e6;
   }

  if (i >= 1e3) {
    if (num >= 1e6)
      printf ("%03d,", (unsigned)(i/1e3));
    else
      printf ("%d,", (unsigned)(i/1e3));

    i -= ((unsigned)(i/1e3)) * 1e3;
    printf ("%03d", i);
   }

  if (num < 1e3) printf ("%d", i);
 }
/*========== End of procedure print_number() ==============================*/


/*========== Returns TRUE is x lies in range y-range to y+range ===========*/
boolean range (const int y, const int range, const int x) {
	/* Simple procedure to determine if a value lies in a particular number
	 * range...can't remember what I use this for, but it could be useful */
  if (x < y - range) return (FALSE);	/* x is less than lower limit */
  if (x > y + range) return (FALSE);	/* x is greater than upper limit */
  return (TRUE);		/* x lies in range */
 }
//========== End of function range() ========================================


//========== Removes our timer interrupt and replaces it with original ======
void remove_timer_int (void) {
	/* This procedure removes our own Timer Interrupt and replaces it
	 * with the original DOS Timer interrupt. Will not replace it if
	 * it has not been previously saved with the set_timer_int() procedure */

  if (Old_Timer_Saved) {
    write_log_entry ("Replacing the Interrupt Timer with Original Routine");

	/* Reset original timer interrupt and boolean variable*/
    _dos_setvect(TIME_KEEPER_INT, Old_Time_Isr);
    Old_Timer_Saved = FALSE;
   }
  else {
	/* Better write a warning notifying the user/programmer. Obviously
	 * there is some bug in the program which results in remove_timer_int()
	 * being called before set_timer_int() */
    write_log_entry ("WARNING: procedure remove_timer_int() called before set_timer_int()!");
   }
 }
//========== End of procedure remove_timer_int() ============================


/*========== Strips Space and TAB characters from Right of String Returns ====
  Pointer to string ========================================================*/
char *rstrip (char *string) {
  char *temp_ptr;

  if (!string) return (NULL);
  temp_ptr = string + strlen(string) - 1;

  while (temp_ptr > string && (*temp_ptr == ' ' || *temp_ptr == 9)) {
    *temp_ptr = 0;
    temp_ptr--;
   }

  return (string); }
/*========== End of function lstrip() =====================================*/


//========== Sets the use of the Timer Interrupt ============================
void set_timer_int(void interrupt (*New_Time_Isr)(__CPPARGS)) {
	/* Save original timer interrupt, but only if we haven't already
	 * saved it.  If we saved it again after replacing it with our own,
	 * we would save the modified timer routine, instead of the original
	 * one....oops */

  if (!Old_Timer_Saved) {
    write_log_entry ("Replacing the Interrupt Timer");
    Old_Time_Isr = _dos_getvect(TIME_KEEPER_INT);
    Old_Timer_Saved = TRUE;
   }
  else {
	/* Better write a warning notifying the user/programer. There really
	 * shouldn't be a need to call this procedure several times */
    write_log_entry ("WARNING: Multiple calls to set_timer_int() !");
   }

	/* Turn the task processing off...initially */
  do_tasks = FALSE;

	/* Replace it with our own timer interrupt function */
  _dos_setvect(TIME_KEEPER_INT, New_Time_Isr);
 }
//========== End of procedure set_timer_int() ===============================


//========== Returns the sign of the Expression (-1/0/1) ====================
int sign (const int i) {
	/* A quick-inline sign routine, returns -1/0/1 depending on the
	 * sign of the expression i. This used to be a define but I changed
	 * it to a function since i is used twice...thus a function will be
	 * faster, esp for complicated/slow expressions...I think */
  return ( i > 0 ? 1 : i == 0 ? 0 : -1 );
 }
//========== End of function sign() =========================================


//========== Initializes the task_list[] array ==============================
void TASK_TYPE::TASK_TYPE (void) {
	/* Constructor for the Class TASK_TYPE. This sets each task to have
	 * some default initial values */
  active = FALSE;	/* Not currently processed */
  max_count = 0; 	/* No delay in task */
  count = 0;	 	/* Reset counter */
  task = NULL;	 	/* No function */
 }
//========== End of procedure init_task_list () =============================


//========== The new Interrupt timer with Multitasking ======================
void interrupt far Timer (__CPPARGS) {
	/* I got the basic idea for this from the book, Teach yourself 'Game
	 * programming in 21 days' although it is changed considerably. This
	 * procedure replaces the original DOS timer interrupt which is, by
	 * default, called 18.2 times per second (this could be modified but
	 * I currently see no reason to). Every time this procedure is called
	 * (called automatically btw), it calls each procedure in the task_list[]
	 * array, and then exits. The procedures called by this routine (these
	 * can be added/deleted with add_task() and delete_task() ) should be
	 * relatively fast ones, not taking up too much time. If they do
	 * take a lot of time, your main game speed will drop dramitically.
	 * The transparent operation of this routine is handy for many things
	 * such as animations (lighting effects, cursor blinking, and colour
	 * rotation are some other things I use it for). Another thing to note,
	 * is that it is not processor specific...so animations and other things
	 * controlled by this  will not be affected by processor speed. You'd
	 * otherwise have to compute the speed before hand and factor that in
	 * to time critical mechanics...ugh! Now you can easily have cursors
	 * that blink the same rate on a 386, as on a 686 (or more) */
  int i;	/* Loop counter for going through the task list*/

	/* Are we processing the multi-tasking procedures currently? This
	 * provides us a handy way of 'pausing' all the actions for adding
	 * tasks, deleting tasks etc... I don't use this variable for pausing
	 * the game action as I may still want to use the multi-tasking environment
	 * for things like blinking cursors or the like. */
  if (do_tasks) {
	/* Turn off the task processing...just in case the called procedures
	 * take a _very_ long time and the Timer Interrupt is called again.
	 * This would result in an infinite loop and the program would appear
	 * to freeze. We turn the task processing back on at the end of this routine */
    do_tasks = FALSE;

	/* Do each task in task_list[] array */
    for (i = 0; i < num_tasks; i++) {
	/* Only do task if it is currently active. This allows us to stop
	 * the execution of some tasks, while still performing others */
      if (task_list[i].active) {

		/* Check if there is a task counter or not */
	if (task_list[i].max_count > 0) {
	  task_list[i].count++;

		/* Is count high enough to trigger task */
	  if (task_list[i].count >= task_list[i].max_count) {
	    task_list[i].task ();
	    task_list[i].count = 0;	//Reset counter
	   }
	 }
	else {
		/* No Task counter...so just execute task */
	  task_list[i].task ();
	 }
       } //End of if task active
     } //End of for i loop
   } //End of if do tasks

/* Now call old interrupt handler. This is only needed if you want to chain
 * interrupts handlers.  In the case that you want total control then the
 * next instruction wouldn't be used. The original timer interrupt routine
 * controls various things like time/date (I think) so we really should chain. */
  Old_Time_Isr();
 }
//========== End Timer ======================================================
